//---------------------------------------------------------------------------
#include <vcl.h>
#include <alloc.h>
#include <math.h>
#pragma hdrstop

#include "core.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)

//----------------------------------------------------------------------------//
// Describes the plugin.                                                      //
// Description: used in the G2 plugin menu and should be shorter than 80      //
//              characters (and ideally less than 30)                         //
// Version    : SDK version used by this plugin.  Should be 2                 //
//----------------------------------------------------------------------------//
DLLExport void DescribePlugin(char* description,int* version)
{
  strcpy(description,"Demonstration PluginII");
  *version = 2;
}

//----------------------------------------------------------------------------//
// Initializes the G2 interfaces.  Passes in function address parameters      //
//----------------------------------------------------------------------------//
DLLExport void InitPlugin(G2_INTERFACE* g2)
{
  G2INTERFACE.GLGGetParameters            = g2->GLGGetParameters;
  G2INTERFACE.GLGSetParameters            = g2->GLGSetParameters;
  G2INTERFACE.GLGCreateNewGrid            = g2->GLGCreateNewGrid;
  G2INTERFACE.GLGGetPointHeight           = g2->GLGGetPointHeight;
  G2INTERFACE.GLGSetPointHeight           = g2->GLGSetPointHeight;
  G2INTERFACE.GLGGetPointWater            = g2->GLGGetPointWater;
  G2INTERFACE.GLGSetPointWater            = g2->GLGSetPointWater;
  G2INTERFACE.GLGGetHeightField           = g2->GLGGetHeightField;
  G2INTERFACE.GLGSetHeightField           = g2->GLGSetHeightField;
  G2INTERFACE.GLGGetTerrainLayerInfo      = g2->GLGGetTerrainLayerInfo;
  G2INTERFACE.GLGSetTerrainLayerInfo      = g2->GLGSetTerrainLayerInfo;
  G2INTERFACE.GLGCreateNewTerrainLayer    = g2->GLGCreateNewTerrainLayer;
  G2INTERFACE.GlGGetPlantInfo             = g2->GlGGetPlantInfo;
  G2INTERFACE.GlGSetPlantInfo             = g2->GlGSetPlantInfo;
  G2INTERFACE.GlGCreateNewPlant           = g2->GlGCreateNewPlant;
  G2INTERFACE.GlGGetUrbanPlan             = g2->GlGGetUrbanPlan;
  G2INTERFACE.GlGSetUrbanPlan             = g2->GlGSetUrbanPlan;
  G2INTERFACE.GlGCreateNewUrbanPlan       = g2->GlGCreateNewUrbanPlan;
  G2INTERFACE.GLGDeleteAllPolygonsInLayer = g2->GLGDeleteAllPolygonsInLayer;
  G2INTERFACE.GLGDeleteAllLinesInLayer    = g2->GLGDeleteAllLinesInLayer;
  G2INTERFACE.GLGDeleteAllModelsInLayer   = g2->GLGDeleteAllModelsInLayer;
  G2INTERFACE.GLGGetViewingParameters     = g2->GLGGetViewingParameters;
  G2INTERFACE.GLGSetViewingParameters     = g2->GLGSetViewingParameters;
  G2INTERFACE.GLGAddModel                 = g2->GLGAddModel;
  G2INTERFACE.GLGGetModelInfo             = g2->GLGGetModelInfo;
  G2INTERFACE.GLGGetPolygonInfo           = g2->GLGGetPolygonInfo;
  G2INTERFACE.GLGGetPolygonCoordinates    = g2->GLGGetPolygonCoordinates;
  G2INTERFACE.GLGAddPolygon               = g2->GLGAddPolygon;
  G2INTERFACE.GLGGetLineInfo              = g2->GLGGetLineInfo;
  G2INTERFACE.GLGGetLineCoordinates       = g2->GLGGetLineCoordinates;
  G2INTERFACE.GLGAddLine                  = g2->GLGAddLine;
  G2INTERFACE.GLGUpdateChildren           = g2->GLGUpdateChildren;
}

//----------------------------------------------------------------------------//
// Run the plugin                                                             //
// This plugin doesn't do anything useful - just manipulates a few parameters //
//----------------------------------------------------------------------------//
DLLExport int RunPlugin()
{
int NewGridWidth;
int NewGridHeight;
int NewGridSpacingX;
int NewGridSpacingY;
int NewTerrainLayerIdx;
int i;
float temp;
int vertexcount;
float *HeightField;
float *NewHeightField;
int pointcount;
int hx,hy;

  // Retrieve basic parameters about the grid
  G2INTERFACE.GLGGetParameters(NULL,&G2PARAMETERS);

  pointcount = G2PARAMETERS.GridWidth*G2PARAMETERS.GridHeight;
  if ((HeightField = (float *)malloc(sizeof(float)*pointcount)) != NULL) {
    G2INTERFACE.GLGGetHeightField(NULL,pointcount,HeightField);
  }

  // Create a new grid - but reduce spacing
  NewGridWidth    = G2PARAMETERS.GridWidth;
  NewGridHeight   = G2PARAMETERS.GridHeight;
  NewGridSpacingX = floor(G2PARAMETERS.GridSpacingX / 2);
  NewGridSpacingY = floor(G2PARAMETERS.GridSpacingY / 2);
  G2INTERFACE.GLGCreateNewGrid(NULL,NewGridWidth,NewGridHeight,NewGridSpacingX,NewGridSpacingY);

  // Mirror image the heightgrid
  if ((HeightField != NULL) &&
     ((NewHeightField = (float *)malloc(sizeof(float)*pointcount)) != NULL)) {
    for(hx=0;hx<G2PARAMETERS.GridWidth;hx++) {
      for(hy=0;hy<G2PARAMETERS.GridHeight;hy++) {
        NewHeightField[(hy*G2PARAMETERS.GridWidth)+hx] = 
           HeightField[(hy*G2PARAMETERS.GridWidth)+(G2PARAMETERS.GridWidth - hx)];
      }
    }
    G2INTERFACE.GLGSetHeightField(NULL,pointcount,NewHeightField);
    free(NewHeightField);
  }
  if (HeightField != NULL) {
    free(HeightField);
  }

  // Manipulate georeferencing information
  temp = G2PARAMETERS.GeoRef[0];
  G2PARAMETERS.GeoRef[0] = G2PARAMETERS.GeoRef[2];
  G2PARAMETERS.GeoRef[2] = temp;
  temp = G2PARAMETERS.GeoRef[1];
  G2PARAMETERS.GeoRef[1] = G2PARAMETERS.GeoRef[3];
  G2PARAMETERS.GeoRef[3] = temp;

  // Retrieve viewing information
  G2INTERFACE.GLGGetViewingParameters(NULL,&G2VIEW);

  // Set parameters as for old grid - but with new georeferencing ingo
  G2INTERFACE.GLGSetParameters(NULL,&G2PARAMETERS);

  // Manipulate some viewing information
  G2VIEW.Perspective /= 1.5;
  G2INTERFACE.GLGSetViewingParameters(NULL,&G2VIEW);

  // If we have a terrain layer defined then we'll do some manipulation
  if (G2PARAMETERS.TerrainLayersCount > 0) {

    // get terrain layer 0 info and duplicate
    G2INTERFACE.GLGGetTerrainLayerInfo(NULL,0,&G2TERRAIN);
    NewTerrainLayerIdx = G2INTERFACE.GLGCreateNewTerrainLayer(NULL,&G2TERRAIN);

    // delete all polygons, lines and models
    for (i=0;i<G2PARAMETERS.TerrainLayersCount;i++) {
      G2INTERFACE.GLGDeleteAllPolygonsInLayer(NULL,i);
      G2INTERFACE.GLGDeleteAllLinesInLayer(NULL,i);
      G2INTERFACE.GLGDeleteAllModelsInLayer(NULL,i);
    }

    // Set up parameters and add a model
    G2MODEL.TerrainLayerID = 0;
    G2MODEL.x = 100;
    G2MODEL.y = 300;
    G2MODEL.Size     = 30;
    G2MODEL.Rotation = 20;
    G2INTERFACE.GLGAddModel(NULL,&G2MODEL);
    G2INTERFACE.GLGGetModelInfo(NULL,0,&G2MODEL);

    // Set up a new polygon
    vertexcount = 5;
    G2POLYGON.TerrainLayerID  = NewTerrainLayerIdx;
    G2POLYGON.CoordinateCount = vertexcount;

    if ((G2POLYGON.Coordinates = (float *)malloc(sizeof(float)*2*G2POLYGON.CoordinateCount)) != NULL) {
      G2POLYGON.Coordinates[0] = 100.0;
      G2POLYGON.Coordinates[1] = 100.0;
      G2POLYGON.Coordinates[2] = 200.0;
      G2POLYGON.Coordinates[3] = 100.0;
      G2POLYGON.Coordinates[4] = 300.0;
      G2POLYGON.Coordinates[5] = 200.0;
      G2POLYGON.Coordinates[6] = 400.0;
      G2POLYGON.Coordinates[7] = 400.0;
      G2POLYGON.Coordinates[8] = 200.0;
      G2POLYGON.Coordinates[9] = 200.0;
      G2INTERFACE.GLGAddPolygon(NULL,&G2POLYGON);
      free(G2POLYGON.Coordinates);
    }
  }

  // Force a child update to reflect the changes we've made
  G2INTERFACE.GLGUpdateChildren(NULL);
  return(1);
}
